import { Kui, PropTypes } from '../kui'
import React from 'react'
// const { Provider, Consumer } = React.createContext('layout')

class Content extends Kui {
  render() {
    const cls = this.className(['k-layout-content'])
    return (<div className={cls} style={this.styles()}>{this.props.children}</div>)
  }
}

class Header extends Kui {
  render() {
    const cls = this.className(['k-layout-header'])
    return (<div className={cls} style={this.styles()}>{this.props.children}</div>)
  }
}

class Footer extends Kui {
  render() {
    const cls = this.className(['k-layout-footer'])
    return (<div className={cls} style={this.styles()}>{this.props.children}</div>)
  }
}

class Sider extends Kui {
  static contextTypes = {
    Layout: PropTypes.any
  }
  componentDidMount() {
    let parent = this.context.Layout
    if (parent) {
      parent.collectSider(1)
    }
  }

  componentWillUnmount() {
    let parent = this.context.Layout
    if (parent) {
      parent.collectSider(0)
    }
  }

  render() {
    const cls = this.className(['k-layout-sider'])
    return (<div className={cls} style={this.styles()}>{this.props.children}</div>)
  }
}


class Layout extends Kui {
  
  static childContextTypes = {
    Layout: PropTypes.any
  }

  state = {
    siders: 0
  }

  collectSider = (ismount) => {
    let { siders } = this.state
    this.setState({ siders: ismount ? (siders + 1) : (siders - 1) })
  }

  getChildContext() {
    return {
      Layout: this
    };
  }

  render() {
    const cls = this.className(['k-layout', {
      'k-layout-has-sider': this.state.siders > 0
    }])

    return (<div className={cls} style={this.styles()}>{this.props.children}</div>)
  }
}

Layout.Sider = Sider
Layout.Content = Content
Layout.Header = Header
Layout.Footer = Footer

export default Layout